/*
  ==============================================================================
  
  Copyright(C) Cyril Russo 2005.

  ------------------------------------------------------------------------------

  This is not aimed to be distributed (too bug prone code)

  ==============================================================================
*/

#include "PinkLemon.h"
#include "Dialog.h"

#define ComponentNumber		12
const char * ComponentRes[] = { "TextEditor", "ComboBox", "Slider", 
                                "TextButton", "ToggleButton", "ArrowButton", 
								"HyperlinkButton", "ColourSelector", "FilenameComponent", 
								"ResizerComponent", "ScrollBar", "TreeView" };




void Dialog::actualize(bool needRepaint)
{
	if (inputWidth)		inputWidth->setText(String(iWidth));
	if (inputHeight)	inputHeight->setText(String(iHeight));
	if (inputX)			inputX->setText(String(iTopLeftX));
	if (inputY)			inputY->setText(String(iTopLeftY));
	if (inputName) 		inputName->setText(Name);
	if (theMainWindow)	theMainWindow->setName(Name);

	if (needRepaint) repaint();
}


//============================================================================
Dialog::Dialog()
 : saveButton(0), quitButton(0), inputWidth(0), inputHeight(0), inputX(0), inputY(0), inputName(0), iWidth(300), iHeight(300), iTopLeftX(0), iTopLeftY(0), theMainWindow(0),
   Name("Dashboard"),
   DialogWindow (T("PinkLemon dialog creator"), 
                    Colours::lavender,
                    false)
					
{
	// Use the default look and feel
	LookAndFeel::setDefaultLookAndFeel(&mainDialogLook);
	// Set the main shape size
	addToDesktop(0);

	int parentWidth = getParentWidth();
	this->centreWithSize(270, 350);
	setSize(270, 350);
	setTopLeftPosition((getParentWidth() - getWidth()) / 2, (getParentHeight() - getHeight()) / 2);
	toFront(true);

	setVisible (true);
    setResizeable (true); // resizability is a property of DialogWindow
	enableDropShadow(true);

    setOpaque (true); 

	// Create the components
	saveButton = new JUCE::TextButton(T("Output"), Colours::black, Colours::darkkhaki, Colours::khaki, "Click here to save your current dialog");
	saveButton->setSize(60, 30);
	saveButton->setTopLeftPosition(getChildPosition(saveButton).getX(), getChildPosition(saveButton).getY());
	addAndMakeVisible(saveButton);
	saveButton->addActionListener(this);

	quitButton = new TextButton(T("Quit"), Colours::black, Colours::darkkhaki, Colours::khaki, "Click here to quit PinkLemon");
	quitButton->setSize(50, 30);
	quitButton->setTopLeftPosition(getChildPosition(quitButton).getX(), getChildPosition(quitButton).getY());
	addAndMakeVisible(quitButton);
	quitButton->addActionListener(this);

	serializeInButton = new TextButton(T("Load"), Colours::black, Colours::khaki, Colours::khaki, "Click here to serialize PinkLemon");
	serializeInButton->setSize(70, 20);
	serializeInButton->setTopLeftPosition(getChildPosition(serializeInButton).getX(), getChildPosition(serializeInButton).getY());
	addAndMakeVisible(serializeInButton);
	serializeInButton->addActionListener(this);

	serializeOutButton = new TextButton(T("Save"), Colours::black, Colours::khaki, Colours::khaki, "Click here to serialize PinkLemon");
	serializeOutButton->setSize(70, 20);
	serializeOutButton->setTopLeftPosition(getChildPosition(serializeOutButton).getX(), getChildPosition(serializeOutButton).getY());
	addAndMakeVisible(serializeOutButton);
	serializeOutButton->addActionListener(this);

	// Create the combo boxes
	componentChooser = new ComboBox("Component");
	componentChooser->setSize(140, 30);
	componentChooser->setTopLeftPosition(getChildPosition(componentChooser).getX(), getChildPosition(componentChooser).getY());
	for (int i = 0; i < ComponentNumber; i++)
	{
		componentChooser->addItem (ComponentRes[i], i);
	}
	componentChooser->setSelectedId(0);
	componentChooser->addActionListener(this);
	addAndMakeVisible(componentChooser);

	addButton = new TextButton(T("Add a"), Colours::black, Colours::khaki, Colours::khaki, "Click here to add PinkLemon");
	addButton->setSize(60, 30);
	addButton->setTopLeftPosition(getChildPosition(addButton).getX(), getChildPosition(addButton).getY());
	addAndMakeVisible(addButton);
	addButton->addActionListener(this);

	inputWidth = new TextEditor("Width");
	inputWidth->setSize(140, 30);
	inputWidth->setTopLeftPosition(getChildPosition(inputWidth).getX(), getChildPosition(inputWidth).getY());
	Label * WidthLabel = new Label(String::empty, "Dialog Width :");
	WidthLabel->attachToComponent(inputWidth, true);
	WidthLabel->setVisible(true);
	inputWidth->setColours(Colours::black, Colours::lavender.brighter(0.06f), Colours::blueviolet.brighter(0.4f), Colours::darkkhaki);
	addAndMakeVisible(inputWidth);

	inputHeight = new TextEditor("Height");
	inputHeight->setSize(140, 30);
	inputHeight->setTopLeftPosition(getChildPosition(inputHeight).getX(), getChildPosition(inputHeight).getY());
	Label * HeightLabel = new Label(String::empty, "Dialog Height :");
	HeightLabel->attachToComponent(inputHeight, true);
	HeightLabel->setVisible(true);
	inputHeight->setColours(Colours::black, Colours::lavender.brighter(0.06f), Colours::blueviolet.brighter(0.4f), Colours::darkkhaki);
	addAndMakeVisible(inputHeight);

	inputX = new TextEditor("X");
	inputX->setSize(140, 30);
	inputX->setTopLeftPosition(getChildPosition(inputX).getX(), getChildPosition(inputX).getY());
	Label * XLabel = new Label(String::empty, "Dialog X :");
	XLabel->attachToComponent(inputX, true);
	XLabel->setVisible(true);
	inputX->setColours(Colours::black, Colours::lavender.brighter(0.06f), Colours::blueviolet.brighter(0.4f), Colours::darkkhaki);
	addAndMakeVisible(inputX);

	inputY = new TextEditor("Y");
	inputY->setSize(140, 30);
	inputY->setTopLeftPosition(getChildPosition(inputY).getX(), getChildPosition(inputY).getY());
	Label * YLabel = new Label(String::empty, "Dialog Y :");
	YLabel->attachToComponent(inputY, true);
	YLabel->setVisible(true);
	inputY->setColours(Colours::black, Colours::lavender.brighter(0.06f), Colours::blueviolet.brighter(0.4f), Colours::darkkhaki);
	addAndMakeVisible(inputY);

	inputName = new TextEditor("Name");
	inputName->setSize(140, 30);
	inputName->setTopLeftPosition(getChildPosition(inputName).getX(), getChildPosition(inputName).getY());
	Label * NameLabel = new Label(String::empty, "Dialog Name :");
	NameLabel->attachToComponent(inputName, true);
	NameLabel->setVisible(true);
	inputName->setColours(Colours::black, Colours::lavender.brighter(0.06f), Colours::blueviolet.brighter(0.4f), Colours::darkkhaki);
	addAndMakeVisible(inputName);
	inputName->addListener(this);


	inputFileName = new TextEditor("Classname");
	inputFileName->setSize(160, 30);
	inputFileName->setTopLeftPosition(getChildPosition(inputFileName).getX(), getChildPosition(inputFileName).getY());
	Label * FileNameLabel = new Label(String::empty, "Class Name :");
	FileNameLabel->attachToComponent(inputFileName, true);
	FileNameLabel->setVisible(true);
	inputFileName->setColours(Colours::black, Colours::lavender.brighter(0.06f), Colours::blueviolet.brighter(0.4f), Colours::darkkhaki);
	addAndMakeVisible(inputFileName);
	inputFileName->addListener(this);

	actualize();
	// Then starts the main window, if needed
	theMainWindow = new Window(this);
//	theMainWindow->addToDesktop(0);
//	addAndMakeVisible(theMainWindow);

}

void  Dialog::textEditorTextChanged (TextEditor &editor)
{
	if (&editor == inputName)
	{
		Name = editor.getText();
		actualize();
	}
}

/** Called when the user presses the return key. */
    void Dialog::textEditorReturnKeyPressed (TextEditor& editor){}

/** Called when the user presses the escape key. */
    void Dialog::textEditorEscapeKeyPressed (TextEditor& editor){}

/** Called when the text editor loses focus. */
    void Dialog::textEditorFocusLost (TextEditor& editor){}


void Dialog::actionListenerCallback(const String &m)
{
	if (m == "Quit")
		JUCEApplication::quit(0);
	if (m == "Add a")
	{
		ComponentInfo * ci = new ComponentInfo();
		ci->componentType = componentChooser->getText();

		theMainWindow->addComponent(ci);
	}
	else if (m == "Output")
	{
		if (inputFileName->getText().length()) theMainWindow->saveTo(inputFileName->getText());
	}
	else if (m == "Load")
	{
		if (inputFileName->getText().length()) 
			if (theMainWindow->serialize(inputFileName->getText() + ".xml", true))
			{
				iTopLeftX = theMainWindow->iTopLeftX;
				iTopLeftY = theMainWindow->iTopLeftY;
				iWidth = theMainWindow->iWidth;
				iHeight = theMainWindow->iHeight;
				Name = theMainWindow->getName();
				actualize();
			}
			else 
			{
				AlertWindow::showMessageBox(AlertWindow::WarningIcon, "Error while reading the dialog file", "The current document format is not understood", "Ok");
			}
		
	}
	else if (m == "Save")
	{
		if (inputFileName->getText().length()) theMainWindow->serialize(inputFileName->getText() + ".xml", false);
	}

	DialogWindow::actionListenerCallback(m);
}


integerPoint Dialog::getChildPosition(Component * child)
{
	integerPoint pt;
	if (child->getName() == "Output")
	{
		pt.setXY(30, getHeight() - 40);
		return pt;
	}
	else if (child->getName() == "Quit")
	{
		pt.setXY(getWidth() - child->getWidth() - 30, getHeight() - 40);
		return pt;
	}
	else if (child->getName() == "Width")
	{
		pt.setXY(getWidth() - child->getWidth() - 10, 40);
	}
	else if (child->getName() == "Height")
	{
		pt.setXY(getWidth() - child->getWidth() - 10, 70);
	}
	else if (child->getName() == "X")
	{
		pt.setXY(getWidth() - child->getWidth() - 10, 100);
	}
	else if (child->getName() == "Y")
	{
		pt.setXY(getWidth() - child->getWidth() - 10, 130);
	}
	else if (child->getName() == "Name")
	{
		pt.setXY(getWidth() - child->getWidth() - 10, 160);
	}
	else if (child->getName() == "Component")
	{
		pt.setXY(getWidth() - child->getWidth() - 10, 200);
	}
	else if (child->getName() == "Add a")
	{
		pt.setXY(40, 200);
	}
	else if (child->getName() == "Classname")
	{
		pt.setXY(getWidth() - child->getWidth() - 10, 250);
	}
	else if (child->getName() == "Load")
	{
		pt.setXY(getWidth() - child->getWidth() - 90, 280);
	}
	else if (child->getName() == "Save")
	{
		pt.setXY(getWidth() - child->getWidth() - 10, 280);
	}

	return pt;
}

void Dialog::resized()
{
	DialogWindow::resized();

	if (saveButton) saveButton->setTopLeftPosition(getChildPosition(saveButton).getX(), getChildPosition(saveButton).getY());
	if (quitButton) quitButton->setTopLeftPosition(getChildPosition(quitButton).getX(), getChildPosition(quitButton).getY());

}

void Dialog::paint(JUCE::Graphics &g)
{
	DialogWindow::paint(g);
//	g.fillAll(Colours::khaki);
}

Dialog::~Dialog()
{
    // This window gets deleted when the user clicks its close button.
	deleteAllChildren();

    // When the window is gone, we need to actually quit the app, so 
    // calling this method does just that:
    JUCEApplication::quit();

    // the content component will get deleted by the DialogWindow base class, 
    // and that will clean up the other demo components.
}

