#ifndef h_CPP_Dialog_CPP_h
#define h_CPP_Dialog_CPP_h


#include "Window.h"

//==============================================================================
class Dialog  : public DialogWindow, public TextEditorListener
{
public:
	// TextEditorListener overrides
public:
	/** Called when the user changes the text in some way. */
    virtual void textEditorTextChanged (TextEditor& editor);

    /** Called when the user presses the return key. */
    virtual void textEditorReturnKeyPressed (TextEditor& editor);

    /** Called when the user presses the escape key. */
    virtual void textEditorEscapeKeyPressed (TextEditor& editor);

    /** Called when the text editor loses focus. */
    virtual void textEditorFocusLost (TextEditor& editor);

	// Action listener callback
public:
    void actionListenerCallback (const String& m);

private:
    // The main window, used as a dashboard
	Window* theMainWindow;
	
	// The width and height
	TextEditor * inputWidth;
	TextEditor * inputHeight;
	TextEditor * inputX;
	TextEditor * inputY;
	TextEditor * inputName;
	

	// The possible component
	ComboBox * componentChooser;
	// The add button
	TextButton * addButton;


	// The serialize button
	TextButton * serializeInButton;
	// The serialize button
	TextButton * serializeOutButton;
	// The save button
	TextButton * saveButton;
	// The quit button
	TextButton * quitButton;

	// The file base name
	TextEditor * inputFileName;

public:
	// Constructor
    Dialog();
	// Destructor
    virtual ~Dialog();

public:
	// Overriding the paint method
    virtual void paint (Graphics& g);
	// And the resized method
	virtual void resized();

public:
	// Returns the child position into the specified window
	integerPoint getChildPosition(Component * child);

	void actualize(bool needRepaint = false);

	inline void setWindowSize(const int & width, const int & height) { iWidth = width; iHeight = height; actualize(); }
	inline void setWindowPosition(const int & x, const int & y)      { iTopLeftX = x; iTopLeftY = y; actualize(); }
	// Initial top left position
	int				 iTopLeftX;
	int				 iTopLeftY;
	// Initial width and height;
	int				 iWidth;
	int				 iHeight;
	// The dialog name
	String			 Name;
};

class MyLookAndFeel : public LookAndFeel
{
public:
    /** Returns the colours to use for various bits of a popup menu. */
    virtual void getPopupMenuColours (Colour& text,
                                      Colour& background,
                                      Colour& highlightText,
                                      Colour& highlightBackground)
	{
		text = Colours::black;
		background = Colours::lavender.brighter(0.06f);
		highlightText = text;
		highlightBackground = Colours::lightblue;
	}

    virtual void getComboBoxColours (Colour& background,
                                     Colour& textColour,
                                     Colour& outlineColour,
                                     Colour& buttonColour,
                                     Colour& popupBackground,
                                     Colour& popupText,
                                     Colour& popupHighlight)
	{
		background       = Colours::lavender.brighter(0.06f);
		textColour       = Colours::black;
		outlineColour    = Colours::darkkhaki;
		buttonColour     = Colours::lightgrey;
		popupBackground  = background;
		popupText        = textColour;
		popupHighlight   = Colours::khaki;
	}

};

static MyLookAndFeel mainDialogLook;


#endif