/*
  ==============================================================================
  
  Copyright(C) Cyril Russo 2005.

  ------------------------------------------------------------------------------

  This is not aimed to be distributed (too bug prone code)

  ==============================================================================
*/

#include "PinkLemon.h"
#include "Window.h"
#include "Dialog.h"

//////////////////////////////////////////////////////////////////////
// Component Info helpers
Component * ComponentInfoVisitor::createComponent(ComponentInfo * ci)
{
	Component * component = 0;
	if (ci)
	{
		if (ci->componentType == "TextEditor")
		{	if (!ci->options.size()) { 	ci->options.add(new bool(false)); ci->options.add(new bool(false)); ci->options.add(new bool(false)); }
			component = new TextEditor(ci->componentName, *(bool*)ci->options[1] ? '' : 0);
			if (component) 
			{
				((TextEditor*)component)->setColours(Colours::black, Colours::white, Colours::khaki, Colours::darkgrey);
				((TextEditor*)component)->setMultiLine(*(bool*)ci->options[0]);
				((TextEditor*)component)->setReadOnly(*(bool*)ci->options[2]);
			}			
		}
		else if (ci->componentType == "ComboBox")
			component = new ComboBox(ci->componentName);
		else if (ci->componentType == "TextButton")
			component = new TextButton(ci->componentName, Colours::black, Colours::khaki, Colours::darkgrey, "Click here");
		else if (ci->componentType == "ToggleButton")
			component = new ToggleButton(ci->componentName);
		else if (ci->componentType == "ArrowButton")
		{	if (!ci->options.size()) { 	ci->options.add(new float(0)); ci->options.add(new ColourUnion(Colours::khaki, "khaki")); }
			ci->size.setXY(jmin(ci->size.getX(), ci->size.getY()),  jmin(ci->size.getX(), ci->size.getY()));
			component = new ArrowButton(ci->componentName, *(float*)ci->options[0], ((ColourUnion*)ci->options[1])->colour);
		}
		else if (ci->componentType == "HyperlinkButton")
			component = new HyperlinkButton(ci->componentName, URL("http://www.rawmaterialsoftwares.com/juce"));
		else if (ci->componentType == "Slider")
			component = new Slider(ci->componentName);
		else if (ci->componentType == "ColourSelector")
			component = new ColourSelector();
		else if (ci->componentType == "FilenameComponent")
			component = new FilenameComponent(ci->componentName, File::getCurrentWorkingDirectory(), true, true, true, "*.*", "*", "Choose your file here");
		else if (ci->componentType == "ResizerComponent")
		{	ci->size.setXY(jmin(ci->size.getX(), ci->size.getY()),  jmin(ci->size.getX(), ci->size.getY()));
			ci->anchorType = ComponentInfo::AnchorRight | ComponentInfo::AnchorBottom;
			component = new ResizerComponent(0);
		}
		else if (ci->componentType == "ScrollBar")
		{  
			component = new ScrollBar((ci->anchorType & ComponentInfo::AnchorBottom) == 0);
		}
		else if (ci->componentType == "TreeView")
			component = new TreeView();

	

		ci->object = component;
	}

	return component;
}

const String ComponentInfoVisitor::getCreationLine(ComponentInfo * ci)
{
	String line = String::empty;
	if (ci)
	{
		if (ci->componentType == "TextEditor")
			line = "TextEditor(T(\"" + ci->componentName + "\"), " + String(*(bool*)ci->options[1] ? "''" : "0") + ");";
		else if (ci->componentType == "ComboBox")
			line = "ComboBox(T(\"" + ci->componentName + "\"));";
		else if (ci->componentType == "TextButton")
			line = "TextButton(T(\"" + ci->componentName + "\"), Colours::black, Colours::khaki, Colours::darkgrey, \"" + ci->componentText + "\");";
		else if (ci->componentType == "ToggleButton")
			line = "ToggleButton(T(\"" + ci->componentName + "\"));";
		else if (ci->componentType == "ArrowButton")
			line = "ArrowButton(T(\"" + ci->componentName + "\"), " + String(*(float*)ci->options[0]) + ", Colours::" + ((ColourUnion*)ci->options[1])->name + ");";
		else if (ci->componentType == "HyperlinkButton")
			line = "HyperlinkButton(T(\"" + ci->componentName + "\"), URL(\"http://www.rawmaterialsoftwares.com/juce\"));";
		else if (ci->componentType == "Slider")
			line = "Slider(T(\"" + ci->componentName + "\"));";
		else if (ci->componentType == "ColourSelector")
			line = "ColourSelector();";
		else if (ci->componentType == "FilenameComponent")
			line = "FilenameComponent(T(\"" + ci->componentName + "\"), File::getCurrentWorkingDirectory(), true, true, true, \"*.*\", \"*\", \"Choose your file here\");";
		else if (ci->componentType == "ResizerComponent")
			line = "ResizerComponent(this);";
		else if (ci->componentType == "ScrollBar")
			line = "ScrollBar(" + String((ci->anchorType & ComponentInfo::AnchorBottom) ? "false" : "true") + ");";
		else if (ci->componentType == "TreeView")
			line = "TreeView();";
	}
	return line;
}

bool ComponentInfoVisitor::startDraggingComponent(ComponentInfo * ci)
{
	if (ci)
	{
		if (ci->componentType == "TextEditor")
		{
			if (ci->object) ((TextEditor*)ci->object)->setColours(Colours::black, Colours::white, Colours::khaki, Colours::red);
		}
		else if (ci->componentType == "TextButton")
		{
			if (ci->object) { *const_cast<Colour*>(&((TextButton*)ci->object)->getBackgroundOutlineColour()) = Colours::red; }
		}
	}

	return true;
}

bool ComponentInfoVisitor::stopDraggingComponent(ComponentInfo * ci)
{
	if (ci)
	{
		if (ci->componentType == "TextEditor")
		{
			if (ci->object) ((TextEditor*)ci->object)->setColours(Colours::black, Colours::white, Colours::khaki, Colours::darkgrey);
		}
		else if (ci->componentType == "TextButton")
		{
			if (ci->object) { *const_cast<Colour*>(&((TextButton*)ci->object)->getBackgroundOutlineColour()) = Colours::darkgrey; }
		}
	}
	return true;
}

int	ComponentInfoVisitor::createAndTrackPopupMenu(ComponentInfo * ci, PopupMenu * pClrMenu)
{
	if (!ci) return 0;
	// Anchor menu first
	PopupMenu subMenu;
	subMenu.addItem (1, "Left", true, (ci->anchorType & ComponentInfo::AnchorLeft) > 0);
    subMenu.addItem (2, "Right", true, (ci->anchorType & ComponentInfo::AnchorRight) > 0);
    subMenu.addItem (3, "Top", true, (ci->anchorType & ComponentInfo::AnchorTop) > 0);
    subMenu.addItem (4, "Bottom", true, (ci->anchorType & ComponentInfo::AnchorBottom) > 0);
    subMenu.addItem (5, "Floating", true, (ci->anchorType & ComponentInfo::AnchorFloating) > 0);

	// Label menu 
	PopupMenu subMenu2;
	subMenu2.addItem (6, "Set label");
	subMenu2.addItem (7, "Put label on left", true, ci->componentLabelLeft);

	PopupMenu m;

	// Add common options first
    m.addSubMenu ("Anchor from", subMenu);
    m.addSubMenu ("Label", subMenu2);
    m.addItem (8, "Set name");
    m.addItem (9, "Set default text");

	// Add specific option
	PopupMenu options;
	if (ci->componentType == "TextEditor")
	{
		options.addItem (100, "Multiline", true, *(bool*)ci->options[0]);
		options.addItem (101, "Password", true, *(bool*)ci->options[1]);
		options.addItem (102, "Read-Only", true, *(bool*)ci->options[2]);
		m.addSubMenu("Options", options);
	} else if (ci->componentType == "ArrowButton")
	{
		options.addItem (100, "Right", true, *(float*)ci->options[0] == 0.0f); 
		options.addItem (101, "Down", true, *(float*)ci->options[0] == 0.25f); 
		options.addItem (102, "Left", true, *(float*)ci->options[0] == 0.5f); 
		options.addItem (103, "Top", true, *(float*)ci->options[0] == 0.75f); 
		options.addSubMenu("Colour", *pClrMenu);
		m.addSubMenu("Options", options);
	}

    const int result = m.show();
	switch (result)
	{
	case 1:
		ci->anchorType ^= ComponentInfo::AnchorLeft;
		ci->anchorType &= ~(ComponentInfo::AnchorFloating | ComponentInfo::AnchorRight);
		break;
	case 2:
		ci->anchorType ^= ComponentInfo::AnchorRight;
		ci->anchorType &= ~(ComponentInfo::AnchorFloating | ComponentInfo::AnchorLeft);
		if ((ci->anchorType & ComponentInfo::AnchorRight) && (ci->componentType == "ScrollBar"))
			((ScrollBar*)ci->object)->setOrientation(true);
		break;
	case 3:
		ci->anchorType ^= ComponentInfo::AnchorTop;
		ci->anchorType &= ~(ComponentInfo::AnchorFloating | ComponentInfo::AnchorBottom);
		break;
	case 4:
		ci->anchorType ^= ComponentInfo::AnchorBottom;
		ci->anchorType &= ~(ComponentInfo::AnchorFloating | ComponentInfo::AnchorTop);
		if ((ci->anchorType & ComponentInfo::AnchorBottom) && (ci->componentType == "ScrollBar"))
			((ScrollBar*)ci->object)->setOrientation(false);
		break;
	case 5:
		ci->anchorType = ComponentInfo::AnchorFloating;
		break;
	case 6:
		{
			String s = "Please set the label of the ";
			s += ci->componentType;
			AlertWindow w("Set component label...", s, AlertWindow::QuestionIcon);

			w.addTextEditor ("text", ci->componentLabel, "New label (empty for none):");

            w.addButton (T("ok"), 1, KeyPress (KeyPress::returnKey, 0));
			w.addButton (T("cancel"), 0, KeyPress (KeyPress::escapeKey, 0));

			if (w.runModalLoop() != 0) // is they picked 'ok'
			{
				// this is the text they entered..
				ci->componentLabel = w.getTextEditorContents (T("text"));
			}
		}
		break;
	case 7:
		ci->componentLabelLeft = !ci->componentLabelLeft; 
		break;
	case 8:
		{
            AlertWindow w (T("Set component name.."),
				T("Please set the name of the ") + ci->componentType,
                            AlertWindow::QuestionIcon);

			w.addTextEditor (T("text"), ci->componentName, T("New name (empty for none):"));

            w.addButton (T("ok"), 1, KeyPress (KeyPress::returnKey, 0));
            w.addButton (T("cancel"), 0, KeyPress (KeyPress::escapeKey, 0));

            if (w.runModalLoop() != 0) // is they picked 'ok'
            {
                // this is the text they entered..
                ci->componentName = w.getTextEditorContents (T("text"));
            }
		}
		break;
	case 9:
		{
            AlertWindow w (T("Set component default text.."),
				T("Please set the default text of the ") + ci->componentType,
                            AlertWindow::QuestionIcon);

			w.addTextEditor (T("text"), ci->componentName, T("New text (empty for none):"));

            w.addButton (T("ok"), 1, KeyPress (KeyPress::returnKey, 0));
            w.addButton (T("cancel"), 0, KeyPress (KeyPress::escapeKey, 0));

            if (w.runModalLoop() != 0) // is they picked 'ok'
            {
                // this is the text they entered..
                ci->componentText = w.getTextEditorContents (T("text"));
				if (ci->componentType == "TextEditor")
				{
					if (ci->object) ((TextEditor*)ci->object)->setText(ci->componentText);
				} else if (ci->componentType == "ComboBox")
				{
					if (ci->object) ((ComboBox*)ci->object)->setText(ci->componentText);
				} else if (ci->componentType == "TextButton")
				{
					if (ci->object) ((TextButton*)ci->object)->setButtonText(ci->componentText);
				}

            }
		}
		break;

	case 0:
	default:
		if (result >= 100)
		{
			if (ci->componentType == "TextEditor")
			{
				switch (result)
				{
				case 100:
					*(bool*)ci->options[0] = !*(bool*)ci->options[0]; 
					if (ci->object)
						((TextEditor*)ci->object)->setMultiLine(*(bool*)ci->options[0]);
					break;
				case 101:
					*(bool*)ci->options[1] = !*(bool*)ci->options[1]; 
//					if (ci->object)
//						((TextEditor*)ci->object)->setMultiline(*(bool*)ci->options[0]);
					break;
				case 102:
					*(bool*)ci->options[2] = !*(bool*)ci->options[2]; 
					if (ci->object)
						((TextEditor*)ci->object)->setReadOnly(*(bool*)ci->options[2]);
					break;
				}
			}
			else if (ci->componentType == "ArrowButton")
			{
				if (result < 1000)
				{
					*(float*)ci->options[0] = ((float)result - 100.0f) * 0.25f;
				}
				else
				{
					*(ColourUnion*)ci->options[1] = Window::arrayClrUnion[result - 1000];
				}
			}
		}

		return 0;

	}

	return 1;
}

ColourUnion findColourUnion(const String & name)
{
	// Find the colour index
	int j = 0;
	for (j = 0; j < sizeof(Window::arrayClrUnion) / sizeof(Window::arrayClrUnion[0]); j++)
	{
		if (name == Window::arrayClrUnion[j].name) return Window::arrayClrUnion[j];
	}

	return Window::arrayClrUnion[0];
}

XmlElement * ComponentInfoVisitor::serialize(ComponentInfo * ci)
{
	if (!ci) return 0;
	XmlElement * element = new XmlElement ("component");
	if (!element) return 0;

	element->setAttribute("type", ci->componentType);
	element->setAttribute("name", ci->componentName);
	element->setAttribute("anchor", ci->anchorType);
	element->setAttribute("text", ci->componentText);
	XmlElement * pos = new XmlElement("visual");
	if (!pos) { delete element; return 0; }
	pos->setAttribute("x", ci->position.getX());
	pos->setAttribute("y", ci->position.getY());
	pos->setAttribute("width", ci->size.getX());
	pos->setAttribute("height", ci->size.getY());
	element->addChildElement(pos);

	XmlElement * label = new XmlElement("label");
	if (!label) { delete element; return 0; }
	label->setAttribute("label", ci->componentLabel);
	label->setAttribute("position", ci->componentLabelLeft ? "left" : "top");
	element->addChildElement(label);

	if (ci->componentType == "TextEditor")
	{
		XmlElement * options = new XmlElement("options");
		if (!options) { delete element; return 0; }
		options->setAttribute("multiline", String(*(bool*)ci->options[0] ? "true" : "false"));
		options->setAttribute("password", String(*(bool*)ci->options[1] ? "true" : "false"));
		options->setAttribute("readonly", String(*(bool*)ci->options[2] ? "true" : "false"));
		element->addChildElement(options);
	}
	else if (ci->componentType == "ArrowButton")
	{
		XmlElement * options = new XmlElement("options");
		if (!options) { delete element; return 0; }
		options->setAttribute("direction", *(float*)ci->options[0]);
		options->setAttribute("colour", ((ColourUnion*)ci->options[1])->name);
		element->addChildElement(options);
	}

	return element;
}

ComponentInfo * ComponentInfoVisitor::serialize(XmlElement * e)
{
	if (!e) return 0;
	// Create the ComponentInfo object
	ComponentInfo * ci = new ComponentInfo();
	if (!ci) return 0; 
	// Then read the element
	if (e->hasTagName("component"))
	{
		ci->componentType = e->getStringAttribute("type");
		ci->componentName = e->getStringAttribute("name");
		ci->componentText = e->getStringAttribute("text");
		ci->anchorType = e->getIntAttribute("anchor");
		
		XmlElement * visual = e->getChildByName("visual");
		if (visual)
		{
			ci->position.setXY(visual->getIntAttribute("x"), visual->getIntAttribute("y"));
			ci->size.setXY(visual->getIntAttribute("width"), visual->getIntAttribute("height"));
		}

		XmlElement * label = e->getChildByName("label");
		if (label)
		{
			ci->componentLabel = label->getStringAttribute("label");
			ci->componentLabelLeft = label->getStringAttribute("position") == "left";
		}

		XmlElement * options = e->getChildByName("options");
		if (options)
		{
			if (ci->componentType == "TextEditor")
			{
				bool bRes = options->getStringAttribute("multiline") == "true";			
				ci->options.add(new bool(bRes));
				bRes = options->getStringAttribute("password") == "true";			
				ci->options.add(new bool(bRes));
				bRes = options->getStringAttribute("readonly") == "true";			
				ci->options.add(new bool(bRes));
			}
			else if (ci->componentType == "ArrowButton")
			{
				float fRes = (float)options->getDoubleAttribute("direction");			
				ci->options.add(new float(fRes));
				ColourUnion clrUnion = findColourUnion(options->getStringAttribute("colour"));
				ci->options.add(new ColourUnion(clrUnion));
			}
		}


	}

	return ci;
}



class CustomMenuComponent  : public PopupMenuCustomComponent, public Timer
{
	ColourUnion clrUnion;
	bool		over;

public:
	CustomMenuComponent(const ColourUnion & clrU) : clrUnion(clrU), over(false)
    {
    }

    ~CustomMenuComponent()
    {
    }

    void getIdealSize (int& idealWidth,
                       int& idealHeight)
    {
        // tells the menu how big we'd like to be..
        idealWidth = 200;
        idealHeight = 15;
    }

    void paint (Graphics& g)
    {
		if (over) g.fillAll (Colour (0xbfa4c2ce));

        g.setColour (clrUnion.colour);
        g.fillRect (1, 1, 20, 13);

        g.setFont (14.0f, Font::italic);
        g.setColour (Colours::black);

        g.drawFittedText (clrUnion.name, 
                          4, 0, getWidth() - 8, getHeight(),
                          Justification::centred, 3);
    }

	virtual void  mouseUp (const MouseEvent &e)
	{
		triggerMenuItem();
		PopupMenuCustomComponent::mouseUp(e);
	}
	virtual void  mouseDrag (const MouseEvent &e)
	{
		PopupMenuCustomComponent::mouseDrag(e);
	}
	virtual void  mouseDown (const MouseEvent &e)
	{
		PopupMenuCustomComponent::mouseDown(e);
	}
	virtual void  mouseMove (const MouseEvent &e)
	{
		timerCallback();
        startTimer (10);
		PopupMenuCustomComponent::mouseMove(e);
	}



    void timerCallback()
    {

        if (! isVisible())
            return;

		if (isMouseOver() != over)
		{
			over = isMouseOver();		
			repaint();

			if (!over) stopTimer();
		}
	}
};

ColourUnion		 Window::arrayClrUnion[151];

//==============================================================================
Window::Window(Dialog * dlg)
    : DialogWindow (T("Dashboard"), 
                    Colours::lavender,
					false), theDialog(dlg), draggedComponent(0), highlightSide(0), resizingComponent(0),
					commandDown(false), gridStep(10)
{
	setVisible (true);
    setResizeable (true); // resizability is a property of DialogWindow
	enableDropShadow(false);

  //  setOpaque (true); 

	iWidth = 300; 	iHeight = 300;
	setSize(iWidth, iHeight);
	iTopLeftX = getScreenX(); iTopLeftY = getScreenY();

	currentBackground.colour = Colours::lavender;
	currentBackground.name = "lavender";

#define MakeComponent(X)	ColourUnion(Colours::X, #X)
	arrayClrUnion[  0] = MakeComponent(transparentBlack);
	arrayClrUnion[  1] = MakeComponent(transparentWhite);
	arrayClrUnion[  2] = MakeComponent(black);
	arrayClrUnion[  3] = MakeComponent(white);
	arrayClrUnion[  4] = MakeComponent(blue);
	arrayClrUnion[  5] = MakeComponent(grey);
	arrayClrUnion[  6] = MakeComponent(green);
	arrayClrUnion[  7] = MakeComponent(red);
	arrayClrUnion[  8] = MakeComponent(yellow);
	arrayClrUnion[  9] = MakeComponent(aliceblue);
	arrayClrUnion[ 10] = MakeComponent(antiquewhite);
	arrayClrUnion[ 12] = MakeComponent(aqua);
	arrayClrUnion[ 13] = MakeComponent(aquamarine);
	arrayClrUnion[ 14] = MakeComponent(azure);
	arrayClrUnion[ 15] = MakeComponent(beige);
	arrayClrUnion[ 16] = MakeComponent(bisque);
	arrayClrUnion[ 17] = MakeComponent(blanchedalmond);
	arrayClrUnion[ 18] = MakeComponent(blueviolet);
	arrayClrUnion[ 19] = MakeComponent(brown);
	arrayClrUnion[ 20] = MakeComponent(burlywood);
	arrayClrUnion[ 21] = MakeComponent(cadetblue);
	arrayClrUnion[ 22] = MakeComponent(chartreuse);
	arrayClrUnion[ 23] = MakeComponent(chocolate);
	arrayClrUnion[ 24] = MakeComponent(coral);
	arrayClrUnion[ 25] = MakeComponent(cornflowerblue);
	arrayClrUnion[ 26] = MakeComponent(cornsilk);
	arrayClrUnion[ 27] = MakeComponent(crimson);
	arrayClrUnion[ 28] = MakeComponent(cyan);
	arrayClrUnion[ 29] = MakeComponent(darkblue);
	arrayClrUnion[ 30] = MakeComponent(darkcyan);
	arrayClrUnion[ 31] = MakeComponent(darkgoldenrod);
	arrayClrUnion[ 32] = MakeComponent(darkgrey);
	arrayClrUnion[ 33] = MakeComponent(darkgreen);
	arrayClrUnion[ 34] = MakeComponent(darkkhaki);
	arrayClrUnion[ 35] = MakeComponent(darkmagenta);
	arrayClrUnion[ 36] = MakeComponent(darkolivegreen);
	arrayClrUnion[ 37] = MakeComponent(darkorange);
	arrayClrUnion[ 38] = MakeComponent(darkorchid);
	arrayClrUnion[ 39] = MakeComponent(darkred);
	arrayClrUnion[ 40] = MakeComponent(darksalmon);
	arrayClrUnion[ 41] = MakeComponent(darkseagreen);
	arrayClrUnion[ 42] = MakeComponent(darkslateblue);
	arrayClrUnion[ 43] = MakeComponent(darkslategrey);
	arrayClrUnion[ 44] = MakeComponent(darkturquoise);
	arrayClrUnion[ 45] = MakeComponent(darkviolet);
	arrayClrUnion[ 46] = MakeComponent(deeppink);
	arrayClrUnion[ 47] = MakeComponent(deepskyblue);
	arrayClrUnion[ 48] = MakeComponent(dimgrey);
	arrayClrUnion[ 49] = MakeComponent(dodgerblue);
	arrayClrUnion[ 50] = MakeComponent(firebrick);
	arrayClrUnion[ 51] = MakeComponent(floralwhite);
	arrayClrUnion[ 52] = MakeComponent(forestgreen);
	arrayClrUnion[ 53] = MakeComponent(fuchsia);
	arrayClrUnion[ 54] = MakeComponent(gainsboro);
	arrayClrUnion[ 55] = MakeComponent(gold);
	arrayClrUnion[ 56] = MakeComponent(goldenrod);
	arrayClrUnion[ 57] = MakeComponent(greenyellow);
	arrayClrUnion[ 58] = MakeComponent(honeydew);
	arrayClrUnion[ 59] = MakeComponent(hotpink);
	arrayClrUnion[ 60] = MakeComponent(indianred);
	arrayClrUnion[ 61] = MakeComponent(indigo);
	arrayClrUnion[ 62] = MakeComponent(ivory);
	arrayClrUnion[ 63] = MakeComponent(khaki);
	arrayClrUnion[ 64] = MakeComponent(lavender);
	arrayClrUnion[ 65] = MakeComponent(lavenderblush);
	arrayClrUnion[ 66] = MakeComponent(lemonchiffon);
	arrayClrUnion[ 67] = MakeComponent(lightblue);
	arrayClrUnion[ 68] = MakeComponent(lightcoral);
	arrayClrUnion[ 69] = MakeComponent(lightcyan);
	arrayClrUnion[ 70] = MakeComponent(lightgoldenrodyellow);
	arrayClrUnion[ 71] = MakeComponent(lightgreen);
	arrayClrUnion[ 72] = MakeComponent(lightgrey);
	arrayClrUnion[ 73] = MakeComponent(lightpink);
	arrayClrUnion[ 74] = MakeComponent(lightsalmon);
	arrayClrUnion[ 75] = MakeComponent(lightseagreen);
	arrayClrUnion[ 76] = MakeComponent(lightskyblue);
	arrayClrUnion[ 77] = MakeComponent(lightslategrey);
	arrayClrUnion[ 78] = MakeComponent(lightsteelblue);
	arrayClrUnion[ 79] = MakeComponent(lightyellow);
	arrayClrUnion[ 80] = MakeComponent(lime);
	arrayClrUnion[ 81] = MakeComponent(limegreen);
	arrayClrUnion[ 82] = MakeComponent(linen);
	arrayClrUnion[ 83] = MakeComponent(magenta);
	arrayClrUnion[ 84] = MakeComponent(maroon);
	arrayClrUnion[ 85] = MakeComponent(mediumaquamarine);
	arrayClrUnion[ 86] = MakeComponent(mediumblue);
	arrayClrUnion[ 87] = MakeComponent(mediumorchid);
	arrayClrUnion[ 88] = MakeComponent(mediumpurple);
	arrayClrUnion[ 89] = MakeComponent(mediumseagreen);
	arrayClrUnion[ 90] = MakeComponent(mediumslateblue);
	arrayClrUnion[ 91] = MakeComponent(mediumspringgreen);
	arrayClrUnion[ 92] = MakeComponent(mediumturquoise);
	arrayClrUnion[ 93] = MakeComponent(mediumvioletred);
	arrayClrUnion[ 94] = MakeComponent(midnightblue);
	arrayClrUnion[ 95] = MakeComponent(mintcream);
	arrayClrUnion[ 96] = MakeComponent(mistyrose);
	arrayClrUnion[ 97] = MakeComponent(navajowhite);
	arrayClrUnion[ 98] = MakeComponent(navy);
	arrayClrUnion[ 99] = MakeComponent(oldlace);
	arrayClrUnion[111] = MakeComponent(olive);
	arrayClrUnion[112] = MakeComponent(olivedrab);
	arrayClrUnion[113] = MakeComponent(orange);
	arrayClrUnion[114] = MakeComponent(orangered);
	arrayClrUnion[115] = MakeComponent(orchid);
	arrayClrUnion[116] = MakeComponent(palegoldenrod);
	arrayClrUnion[117] = MakeComponent(palegreen);
	arrayClrUnion[118] = MakeComponent(paleturquoise);
	arrayClrUnion[119] = MakeComponent(palevioletred);
	arrayClrUnion[120] = MakeComponent(papayawhip);
	arrayClrUnion[121] = MakeComponent(peachpuff);
	arrayClrUnion[122] = MakeComponent(peru);
	arrayClrUnion[123] = MakeComponent(pink);
	arrayClrUnion[124] = MakeComponent(plum);
	arrayClrUnion[125] = MakeComponent(powderblue);
	arrayClrUnion[126] = MakeComponent(purple);
	arrayClrUnion[127] = MakeComponent(rosybrown);
	arrayClrUnion[128] = MakeComponent(royalblue);
	arrayClrUnion[129] = MakeComponent(saddlebrown);
	arrayClrUnion[130] = MakeComponent(salmon);
	arrayClrUnion[131] = MakeComponent(sandybrown);
	arrayClrUnion[132] = MakeComponent(seagreen);
	arrayClrUnion[133] = MakeComponent(seashell);
	arrayClrUnion[134] = MakeComponent(sienna);
	arrayClrUnion[135] = MakeComponent(silver);
	arrayClrUnion[136] = MakeComponent(skyblue);
	arrayClrUnion[137] = MakeComponent(slateblue);
	arrayClrUnion[138] = MakeComponent(slategrey);
	arrayClrUnion[139] = MakeComponent(snow);
	arrayClrUnion[140] = MakeComponent(springgreen);
	arrayClrUnion[141] = MakeComponent(steelblue);
	arrayClrUnion[142] = MakeComponent(tan);
	arrayClrUnion[143] = MakeComponent(teal);
	arrayClrUnion[144] = MakeComponent(thistle);
	arrayClrUnion[145] = MakeComponent(tomato);
	arrayClrUnion[146] = MakeComponent(turquoise);
	arrayClrUnion[147] = MakeComponent(violet);
	arrayClrUnion[148] = MakeComponent(wheat);
	arrayClrUnion[149] = MakeComponent(whitesmoke);
	arrayClrUnion[150] = MakeComponent(yellowgreen);


#define MakeItem(X)	1000 + X, new CustomMenuComponent(arrayClrUnion[X])
	clrMenu.addCustomItem (MakeItem(  0));    //   transparentBlack
	clrMenu.addCustomItem (MakeItem(  1));    //   transparentWhite
	clrMenu.addCustomItem (MakeItem(  2));    //   black
	clrMenu.addCustomItem (MakeItem(  3));    //   white
	clrMenu.addCustomItem (MakeItem(  4));    //   blue
	clrMenu.addCustomItem (MakeItem(  5));    //   grey
	clrMenu.addCustomItem (MakeItem(  6));    //   green
	clrMenu.addCustomItem (MakeItem(  7));    //   red
	clrMenu.addCustomItem (MakeItem(  8));    //   yellow
	clrMenu.addCustomItem (MakeItem(  9));    //   aliceblue
	clrMenu.addCustomItem (MakeItem( 10));    //   antiquewhite
	clrMenu.addCustomItem (MakeItem( 12));    //   aqua
	clrMenu.addCustomItem (MakeItem( 13));    //   aquamarine
	clrMenu.addCustomItem (MakeItem( 14));    //   azure
	clrMenu.addCustomItem (MakeItem( 15));    //   beige
	clrMenu.addCustomItem (MakeItem( 16));    //   bisque
	clrMenu.addCustomItem (MakeItem( 17));    //   blanchedalmond
	clrMenu.addCustomItem (MakeItem( 18));    //   blueviolet
	clrMenu.addCustomItem (MakeItem( 19));    //   brown
	clrMenu.addCustomItem (MakeItem( 20));    //   burlywood
	clrMenu.addCustomItem (MakeItem( 21));    //   cadetblue
	clrMenu.addCustomItem (MakeItem( 22));    //   chartreuse
	clrMenu.addCustomItem (MakeItem( 23));    //   chocolate
	clrMenu.addCustomItem (MakeItem( 24));    //   coral
	clrMenu.addCustomItem (MakeItem( 25));    //   cornflowerblue
	clrMenu.addCustomItem (MakeItem( 26));    //   cornsilk
	clrMenu.addCustomItem (MakeItem( 27));    //   crimson
	clrMenu.addCustomItem (MakeItem( 28));    //   cyan
	clrMenu.addCustomItem (MakeItem( 29));    //   darkblue
	clrMenu.addCustomItem (MakeItem( 30));    //   darkcyan
	clrMenu.addCustomItem (MakeItem( 31));    //   darkgoldenrod
	clrMenu.addCustomItem (MakeItem( 32));    //   darkgrey
	clrMenu.addCustomItem (MakeItem( 33));    //   darkgreen
	clrMenu.addCustomItem (MakeItem( 34));    //   darkkhaki
	clrMenu.addCustomItem (MakeItem( 35));    //   darkmagenta
	clrMenu.addCustomItem (MakeItem( 36));    //   darkolivegreen
	clrMenu.addCustomItem (MakeItem( 37));    //   darkorange
	clrMenu.addCustomItem (MakeItem( 38));    //   darkorchid
	clrMenu.addCustomItem (MakeItem( 39));    //   darkred
	clrMenu.addCustomItem (MakeItem( 40));    //   darksalmon
	clrMenu.addCustomItem (MakeItem( 41));    //   darkseagreen
	clrMenu.addCustomItem (MakeItem( 42));    //   darkslateblue
	clrMenu.addCustomItem (MakeItem( 43));    //   darkslategrey
	clrMenu.addCustomItem (MakeItem( 44));    //   darkturquoise
	clrMenu.addCustomItem (MakeItem( 45));    //   darkviolet
	clrMenu.addCustomItem (MakeItem( 46));    //   deeppink
	clrMenu.addCustomItem (MakeItem( 47));    //   deepskyblue
	clrMenu.addCustomItem (MakeItem( 48));    //   dimgrey
	clrMenu.addCustomItem (MakeItem( 49));    //   dodgerblue
	clrMenu.addCustomItem (MakeItem( 50));    //   firebrick
	clrMenu.addCustomItem (MakeItem( 51));    //   floralwhite
	clrMenu.addCustomItem (MakeItem( 52));    //   forestgreen
	clrMenu.addCustomItem (MakeItem( 53));    //   fuchsia
	clrMenu.addCustomItem (MakeItem( 54));    //   gainsboro
	clrMenu.addCustomItem (MakeItem( 55));    //   gold
	clrMenu.addCustomItem (MakeItem( 56));    //   goldenrod
	clrMenu.addCustomItem (MakeItem( 57));    //   greenyellow
	clrMenu.addCustomItem (MakeItem( 58));    //   honeydew
	clrMenu.addCustomItem (MakeItem( 59));    //   hotpink
	clrMenu.addCustomItem (MakeItem( 60));    //   indianred
	clrMenu.addCustomItem (MakeItem( 61));    //   indigo
	clrMenu.addCustomItem (MakeItem( 62));    //   ivory
	clrMenu.addCustomItem (MakeItem( 63));    //   khaki
	clrMenu.addCustomItem (MakeItem( 64));    //   lavender
	clrMenu.addCustomItem (MakeItem( 65));    //   lavenderblush
	clrMenu.addCustomItem (MakeItem( 66));    //   lemonchiffon
	clrMenu.addCustomItem (MakeItem( 67));    //   lightblue
	clrMenu.addCustomItem (MakeItem( 68));    //   lightcoral
	clrMenu.addCustomItem (MakeItem( 69));    //   lightcyan
	clrMenu.addCustomItem (MakeItem( 70));    //   lightgoldenrodyellow
	clrMenu.addCustomItem (MakeItem( 71));    //   lightgreen
	clrMenu.addCustomItem (MakeItem( 72));    //   lightgrey
	clrMenu.addCustomItem (MakeItem( 73));    //   lightpink
	clrMenu.addCustomItem (MakeItem( 74));    //   lightsalmon
	clrMenu.addCustomItem (MakeItem( 75));    //   lightseagreen
	clrMenu.addCustomItem (MakeItem( 76));    //   lightskyblue
	clrMenu.addCustomItem (MakeItem( 77));    //   lightslategrey
	clrMenu.addCustomItem (MakeItem( 78));    //   lightsteelblue
	clrMenu.addCustomItem (MakeItem( 79));    //   lightyellow
	clrMenu.addCustomItem (MakeItem( 80));    //   lime
	clrMenu.addCustomItem (MakeItem( 81));    //   limegreen
	clrMenu.addCustomItem (MakeItem( 82));    //   linen
	clrMenu.addCustomItem (MakeItem( 83));    //   magenta
	clrMenu.addCustomItem (MakeItem( 84));    //   maroon
	clrMenu.addCustomItem (MakeItem( 85));    //   mediumaquamarine
	clrMenu.addCustomItem (MakeItem( 86));    //   mediumblue
	clrMenu.addCustomItem (MakeItem( 87));    //   mediumorchid
	clrMenu.addCustomItem (MakeItem( 88));    //   mediumpurple
	clrMenu.addCustomItem (MakeItem( 89));    //   mediumseagreen
	clrMenu.addCustomItem (MakeItem( 90));    //   mediumslateblue
	clrMenu.addCustomItem (MakeItem( 91));    //   mediumspringgreen
	clrMenu.addCustomItem (MakeItem( 92));    //   mediumturquoise
	clrMenu.addCustomItem (MakeItem( 93));    //   mediumvioletred
	clrMenu.addCustomItem (MakeItem( 94));    //   midnightblue
	clrMenu.addCustomItem (MakeItem( 95));    //   mintcream
	clrMenu.addCustomItem (MakeItem( 96));    //   mistyrose
	clrMenu.addCustomItem (MakeItem( 97));    //   navajowhite
	clrMenu.addCustomItem (MakeItem( 98));    //   navy
	clrMenu.addCustomItem (MakeItem( 99));    //   oldlace
	clrMenu.addCustomItem (MakeItem(111));    //   olive
	clrMenu.addCustomItem (MakeItem(112));    //   olivedrab
	clrMenu.addCustomItem (MakeItem(113));    //   orange
	clrMenu.addCustomItem (MakeItem(114));    //   orangered
	clrMenu.addCustomItem (MakeItem(115));    //   orchid
	clrMenu.addCustomItem (MakeItem(116));    //   palegoldenrod
	clrMenu.addCustomItem (MakeItem(117));    //   palegreen
	clrMenu.addCustomItem (MakeItem(118));    //   paleturquoise
	clrMenu.addCustomItem (MakeItem(119));    //   palevioletred
	clrMenu.addCustomItem (MakeItem(120));    //   papayawhip
	clrMenu.addCustomItem (MakeItem(121));    //   peachpuff
	clrMenu.addCustomItem (MakeItem(122));    //   peru
	clrMenu.addCustomItem (MakeItem(123));    //   pink
	clrMenu.addCustomItem (MakeItem(124));    //   plum
	clrMenu.addCustomItem (MakeItem(125));    //   powderblue
	clrMenu.addCustomItem (MakeItem(126));    //   purple
	clrMenu.addCustomItem (MakeItem(127));    //   rosybrown
	clrMenu.addCustomItem (MakeItem(128));    //   royalblue
	clrMenu.addCustomItem (MakeItem(129));    //   saddlebrown
	clrMenu.addCustomItem (MakeItem(130));    //   salmon
	clrMenu.addCustomItem (MakeItem(131));    //   sandybrown
	clrMenu.addCustomItem (MakeItem(132));    //   seagreen
	clrMenu.addCustomItem (MakeItem(133));    //   seashell
	clrMenu.addCustomItem (MakeItem(134));    //   sienna
	clrMenu.addCustomItem (MakeItem(135));    //   silver
	clrMenu.addCustomItem (MakeItem(136));    //   skyblue
	clrMenu.addCustomItem (MakeItem(137));    //   slateblue
	clrMenu.addCustomItem (MakeItem(138));    //   slategrey
	clrMenu.addCustomItem (MakeItem(139));    //   snow
	clrMenu.addCustomItem (MakeItem(140));    //   springgreen
	clrMenu.addCustomItem (MakeItem(141));    //   steelblue
	clrMenu.addCustomItem (MakeItem(142));    //   tan
	clrMenu.addCustomItem (MakeItem(143));    //   teal
	clrMenu.addCustomItem (MakeItem(144));    //   thistle
	clrMenu.addCustomItem (MakeItem(145));    //   tomato
	clrMenu.addCustomItem (MakeItem(146));    //   turquoise
	clrMenu.addCustomItem (MakeItem(147));    //   violet
	clrMenu.addCustomItem (MakeItem(148));    //   wheat
	clrMenu.addCustomItem (MakeItem(149));    //   whitesmoke
	clrMenu.addCustomItem (MakeItem(150));    //   yellowgreen
#undef MakeComponent
#undef MakeItem
}

void Window::keyPressed(const KeyPress & key)
{
	DialogWindow::keyPressed(key);
}


void Window::modifierKeysChanged(const ModifierKeys & modifiers)
{
	if (modifiers.isCommandDown() != commandDown)
	{
		commandDown = modifiers.isCommandDown();
		repaint();
	}
}

void Window::focusGained (FocusChangeType cause)
{
}

void Window::broughtToFront()
{
	// Set the z-order for all windows
//	this->toFront(true);
}



void Window::paint (Graphics& g)
{
	DialogWindow::paint(g);

	g.setColour(Colours::red);
	switch (highlightSide)
	{
		// Left
	case 1:
	case 5:
	case 9:
		g.drawLine((float)1, (float)g.getClipTop(), (float)1, (float)g.getClipBottom(), (float)3);
		break;

		// Right
	case 2:
	case 6:
	case 10:
		g.drawLine((float)getWidth() - 3, (float)g.getClipTop(), (float)getWidth() - 3, (float)g.getClipBottom(), (float)3);
		break;
	default:
		break;
	}
	switch (highlightSide)
	{
		// Top
	case 4:
	case 5:
	case 6:
		g.drawLine((float)g.getClipLeft(), (float)1, (float)g.getClipRight(), (float)1, (float)3);
		break;

		// Bottom
	case 8:
	case 9:
	case 10:
		g.drawLine((float)g.getClipLeft(), (float)getHeight() - 3, (float)g.getClipRight(), (float)getHeight() - 3, (float)3);
		break;

	case 0:
	default:
		break;
	}

	// Show the grid now (if needed)
	if (commandDown)
	{
		g.setColour(Colours::grey);
		int i = 0, j = 0;
		for (j = 0; j < g.getClipHeight(); j += gridStep)
			for (i = 0; i < g.getClipWidth(); i+= gridStep)
			{
				g.setPixel(i, j);
			}
	}
//	g.fillAll(Colours::white);
}

Window::~Window()
{

	deleteAllChildren();

    // When the window is gone, we need to actually quit the app, so 
    // calling this method does just that:
    JUCEApplication::quit();

    // the content component will get deleted by the DialogWindow base class, 
    // and that will clean up the other demo arrayComponents.
}

void Window::resized()
{
	if (theDialog) theDialog->setWindowSize(getWidth(), getHeight());
	DialogWindow::resized();

	// Get the difference with the current resize
	int dx = getScreenX() - iTopLeftX, dy = getScreenY() - iTopLeftY;
	if (dx || dy)
	{
		for (int i = 0; i < arrayComponents.size(); i++)
		{
			// We have to invert the DialogWindow top left anchor effect for top left anchors only (only if right/bottom anchors are not set)
			arrayComponents[i]->object->setTopLeftPosition(arrayComponents[i]->position.getX() - dx * ((arrayComponents[i]->anchorType & ComponentInfo::AnchorRight) == 0), arrayComponents[i]->position.getY() - dy * ((arrayComponents[i]->anchorType & ComponentInfo::AnchorBottom) == 0));			
			arrayComponents[i]->position.translate(-dx * ((arrayComponents[i]->anchorType & ComponentInfo::AnchorRight) == 0), -dy * ((arrayComponents[i]->anchorType & ComponentInfo::AnchorBottom) == 0));


			if (arrayComponents[i]->anchorType & ComponentInfo::AnchorLeft || arrayComponents[i]->anchorType & ComponentInfo::AnchorTop)
			{
				int bLeft = (arrayComponents[i]->anchorType & ComponentInfo::AnchorLeft) > 0;
				int bTop = (arrayComponents[i]->anchorType & ComponentInfo::AnchorTop) > 0;

				arrayComponents[i]->object->setTopLeftPosition(arrayComponents[i]->position.getX() + dx * bLeft, arrayComponents[i]->position.getY() + dy * bTop);			
				arrayComponents[i]->position.translate(dx * bLeft, dy * bTop);
			}

		}

		iTopLeftX += dx;
		iTopLeftY += dy;
	}

	dx = getWidth() - iWidth, dy = getHeight() - iHeight;
	if (dx || dy)
	{
		for (int i = 0; i < arrayComponents.size(); i++)
		{
			if (arrayComponents[i]->anchorType & ComponentInfo::AnchorRight || arrayComponents[i]->anchorType & ComponentInfo::AnchorBottom)
			{
				int bRight = (arrayComponents[i]->anchorType & ComponentInfo::AnchorRight) > 0;
				int bBottom = (arrayComponents[i]->anchorType & ComponentInfo::AnchorBottom) > 0;
				arrayComponents[i]->object->setTopLeftPosition(arrayComponents[i]->position.getX() + dx * bRight, arrayComponents[i]->position.getY() + dy * bBottom);			
				arrayComponents[i]->position.translate(dx * bRight, dy * bBottom);
			}
		}

		iWidth += dx;
		iHeight += dy;
	}

}

void Window::moved()
{
	if (theDialog) theDialog->setWindowPosition(this->getX(), this->getY());

	DialogWindow::moved();
	// When resizing from the top and/or left corner, a move message is send before a resized message
	// There is no interface which could tell if we are resizing or moving
	// So I'm using a hack, as if we are resizing, the cursor is changed so we can detect it
	if (getMouseCursor() == MouseCursor::NormalCursor) { iTopLeftX = getScreenX(); iTopLeftY = getScreenY(); }
}

void Window::addComponent(ComponentInfo * ci)
{
	if (ci)
	{
		if (ci->componentName == "Cmp_")
			ci->componentName += String(arrayComponents.size() + 1);
		Component * component = ComponentInfoVisitor::createComponent(ci);

		if (component)
		{
			component->setBounds(ci->position.getX(), ci->position.getY(), ci->size.getX(), ci->size.getY());
			component->setInterceptsMouseClicks(false);
			addAndMakeVisible(component);
			arrayComponents.add(ci);
		}
	}
}

ComponentInfo * Window::findComponent(const integerPoint & pt, const int & deviation)
{
	for (int i = 0; i < arrayComponents.size(); i++)
	{
		if (arrayComponents[i]->position.getX() <= pt.getX() && arrayComponents[i]->position.getY() <= pt.getY()
		  && (arrayComponents[i]->position.getX() + arrayComponents[i]->size.getX() + deviation) >= pt.getX() && (arrayComponents[i]->position.getY() + arrayComponents[i]->size.getY()) >= pt.getY())
			return arrayComponents[i];
	}

	return 0;
}

void Window::mouseDown (const MouseEvent& e2)
{
	integerPoint pt(e2.x, e2.y);
	ComponentInfo * c = findComponent(pt, 5);
	if (c && e2.mods.isLeftButtonDown())
	{
		int a = 0;

		if (c->position.getX() + c->size.getX() - 7 < e2.x)
		{
			resizingComponent = c;
			resizingOrigin.setXY(e2.x, e2.y);
			resizingOriginSize = c->size;
		} else
		{
			draggedComponent = c;
		}

		// For grid purpose
		draggedSrc.setXY((e2.x / gridStep) * gridStep, (e2.y / gridStep) * gridStep);

		ComponentInfoVisitor::startDraggingComponent(c);
		if (c->anchorType != ComponentInfo::AnchorFloating)
		{
			highlightSide = c->anchorType;
		} else highlightSide = 0;

		repaint();
	} else
		DialogWindow::mouseDown(e2);
}

void Window::mouseDrag (const MouseEvent& e2)
{
	integerPoint pt(e2.x, e2.y);
	if (draggedComponent)
	{
		if (commandDown && draggedComponent->object)
		{
			int x = ((e2.x - draggedSrc.getX()) / gridStep) * gridStep;
			int y = ((e2.y - draggedSrc.getY()) / gridStep) * gridStep;
			draggedComponent->object->setTopLeftPosition(draggedSrc.getX() + x, draggedSrc.getY() + y);
			draggedComponent->position.setXY(draggedSrc.getX() + x, draggedSrc.getY() + y);
			repaint();
		} else if (draggedComponent->object) draggedComponent->object->setTopLeftPosition(e2.x, e2.y);
	} else	if (resizingComponent)
	{
		if (commandDown && resizingComponent->object)
		{
			int x = ((e2.x - resizingOrigin.getX() + resizingOriginSize.getX()) / gridStep) * gridStep;
			int y = ((e2.y - resizingOrigin.getY() + resizingOriginSize.getY()) / gridStep) * gridStep;
			resizingComponent->object->setSize(x, y);
			resizingComponent->size.setXY(x, y);
			repaint();
		} else if (resizingComponent->object) resizingComponent->object->setSize(e2.x - resizingOrigin.getX() + resizingOriginSize.getX(), e2.y - resizingOrigin.getY() + resizingOriginSize.getY());
	}
	else
		DialogWindow::mouseDrag(e2);
}

void Window::mouseMove (const MouseEvent& e2)
{
	integerPoint pt(e2.x, e2.y);
	ComponentInfo * c = findComponent(pt, 5);
	static ComponentInfo * oldc = 0;

	if (c)
	{
		if (c->position.getX() + c->size.getX() - 5 < e2.x)
		{
			setMouseCursor(MouseCursor::BottomRightCornerResizeCursor);
			oldc = c;
		} else
		{
			setMouseCursor(MouseCursor::NormalCursor);
			oldc = 0;
		}
	} 
	else if (oldc)
	{
		setMouseCursor(MouseCursor::NormalCursor);
		oldc = 0;
	}

	DialogWindow::mouseMove(e2);
}

void Window::mouseUp (const MouseEvent& e2)
{
	if (draggedComponent)
	{
		int x = ((e2.x - draggedSrc.getX()) / gridStep) * gridStep + draggedSrc.getX();
		int y = ((e2.y - draggedSrc.getY()) / gridStep) * gridStep + draggedSrc.getY();
		draggedComponent->position.setXY(commandDown ? x : e2.x, commandDown ? y : e2.y);	
		ComponentInfoVisitor::stopDraggingComponent(draggedComponent);
		draggedComponent = 0;
		highlightSide = 0;

		repaint();
	}
	else if (resizingComponent)
	{
		int x = ((e2.x - resizingOrigin.getX() + resizingOriginSize.getX()) / gridStep) * gridStep;
		int y = ((e2.y - resizingOrigin.getY() + resizingOriginSize.getY()) / gridStep) * gridStep;
		resizingComponent->size.setXY(commandDown ? x : (e2.x - resizingOrigin.getX() + resizingOriginSize.getX()), commandDown ? y : (e2.y - resizingOrigin.getY() + resizingOriginSize.getY()));
		ComponentInfoVisitor::stopDraggingComponent(resizingComponent);
		resizingComponent = 0;

		repaint();
	}
	else
	{
		integerPoint pt(e2.x, e2.y);
		ComponentInfo * c = findComponent(pt);
		if (c && e2.mods.isPopupMenu())
		{
			int choice = ComponentInfoVisitor::createAndTrackPopupMenu(c, &clrMenu);
			if (choice)
			{
				if (c->label) deleteAndZero(c->label);

				if (c->componentLabel.length() )
				{
					c->label = new Label(String::empty, c->componentLabel);
					c->label->attachToComponent(c->object, c->componentLabelLeft);
					c->label->setVisible(true);
				} 
					
				if (c->object) c->object->setName(c->componentName);
			}

		} else
		{
			if (e2.mods.isPopupMenu())
			{
				// Create the default popup menu
				// Grid size menu first
				PopupMenu subMenu;
				subMenu.addItem (2, "2 px", true, gridStep == 2);
				subMenu.addItem (5, "5 px", true, gridStep == 5);
				subMenu.addItem (10, "10 px", true, gridStep == 10);
				subMenu.addItem (15, "15 px", true, gridStep == 15);
				subMenu.addItem (20, "20 px", true, gridStep == 20);

				// Then menu itself
				PopupMenu rcMenu;
				rcMenu.addSubMenu("Grid size", subMenu);
				rcMenu.addSubMenu("Default colour scheme", clrMenu);

				int res = rcMenu.show();
				if (res && res < 1000)
				{
					gridStep = res;
					repaint();
				}
				else if (res > 1000)
				{
					// Current background is now arrayClrUnion[res - 1000]
					// Ugly hack to get address of a private member from DialogWindow
					// First find the current colour address
					changeBackgroundColor(arrayClrUnion[res - 1000].name); 
					repaint();
				}
			}
			else
				DialogWindow::mouseUp(e2);
		}
	}
}

void Window::changeBackgroundColor(const String & name)
{
	// Find the colour index
	int j = 0;
	for (j = 0; j < sizeof(arrayClrUnion) / sizeof(arrayClrUnion[0]); j++)
	{
		if (name == arrayClrUnion[j].name) break;
	}

	if (j < sizeof(arrayClrUnion) / sizeof(arrayClrUnion[0]))
	{
		int i = 0; int * pthis = (int*)this; 
		for (; i < sizeof(*this); i += 4)
		{
			if (memcmp (pthis, &currentBackground.colour, 4) == 0)
			{
				// Found
				break;
			}
			pthis ++;
		}
		currentBackground = arrayClrUnion[j];
		// This is very ugly
		*pthis = *(int*)&currentBackground.colour;
	}
}

int Window::getComponentIndex(const String & type, const int & start)
{
	for (int i = start; i < arrayComponents.size(); i++)
	{
		if (arrayComponents[i]->componentType == type)
			return i;
	}
	return -1;	
}

void Window::saveTo(const String & baseName)
{
	try
	{
		int i = 0;
		// Okay, now, create the 2 files for outputting stuff in them
		File header(File::getCurrentWorkingDirectory().getFullPathName() + "\\" +  baseName + ".hpp");
		// Create it if it doesn't exists
		if (header.exists()) header.deleteFile();
		header.create();

		// Start by adding some comments to it
		header.appendText("/********************** " + getName() + " dialog declaration ****************\n");
		header.appendText("| This file declares " + getName() + " dialog. Created with Pink Lemon!      \n");
		header.appendText("| \n");
		header.appendText("| Feel free to modify. This file is under GPL license.\n");
		header.appendText("\\********************** " + getName() + " dialog declaration ***************/\n\n");

		// Prevent double inclusion
		header.appendText("#ifndef hpp_CPP_" + baseName.removeCharacters(" ") + "_CPP_hpp\n");
		header.appendText("#define hpp_CPP_" + baseName.removeCharacters(" ") + "_CPP_hpp\n\n\n");

		// Now, create the class
		header.appendText("class " + baseName.removeCharacters(" ") + " : public DialogWindow");
		if (getComponentIndex("TextEditor") != -1) 
			header.appendText(", public TextEditorListener");
		if (getComponentIndex("ScrollBar") != -1) 
			header.appendText(", public ChangeListener");
		// End of base classes
		header.appendText("\n{\n");

		if (getComponentIndex("TextEditor") != -1) 
			header.appendText("public:\n	// TextEditorListener overrides\n\
public:\n\
\t/** Called when the user changes the text in some way. */\n\
\tvirtual void textEditorTextChanged (TextEditor& editor);\n\
\n\
\t/** Called when the user presses the return key. */\n\
\tvirtual void textEditorReturnKeyPressed (TextEditor& editor);\n\
\n\
\t/** Called when the user presses the escape key. */\n\
\tvirtual void textEditorEscapeKeyPressed (TextEditor& editor);\n\
\n\
\t/** Called when the text editor loses focus. */\n\
\tvirtual void textEditorFocusLost (TextEditor& editor);\n\n");

		// Scrollbar or ColourSelector component
		if (getComponentIndex("ScrollBar") != -1 || getComponentIndex("ColourSelector") != -1) 
			header.appendText("public:\n	// Scrollbar or/and ColourSelector overrides\n\
public:\n\
\t/** Called when scrolled or selected a colour the window. */\n\
\tvirtual void  changeListenerCallback (void *objectThatHasChanged);\n\n");


		// Add the action listener callback
		header.appendText("public:\n\t// Action listener callback\n\tvoid actionListenerCallback (const String& m);\n\n");

		// Create the members
		header.appendText("private:\n	// Member declarations\n");
		for (i = 0; i < arrayComponents.size(); i++)
		{
			header.appendText("\t/** The "+ arrayComponents[i]->componentName + " " + arrayComponents[i]->componentType + " */\n");
			header.appendText("\t"+ arrayComponents[i]->componentType + " * \t" + arrayComponents[i]->componentName.removeCharacters(" ") + ";\n");
		}

		// Write down the constructors and destructors
		header.appendText("public:\n	// Constructor and destructor\n");
		header.appendText("\t" + baseName.removeCharacters(" ") + "();\n");
		header.appendText("\tvirtual ~" + baseName.removeCharacters(" ") + "();\n\n");

		header.appendText("public:\n	// Overrides\n");
		header.appendText("\t/** Overriding the paint method for own drawing */\n");
		header.appendText("\tvirtual void paint (Graphics& g);\n");
		header.appendText("\t/** Overriding the resized method for anchoring */\n");
		header.appendText("\tvirtual void resized();\n\n");

		// Then create the auxiliary methods (for anchoring)
		header.appendText("private:\n	// Get child position with anchoring applied\n");
		header.appendText("\t/** Returns the given child position into the dialog */\n");
		header.appendText("\tintegerPoint getChildPosition(Component * child);\n\n");

		// Now finish the dialog class
		header.appendText("};\n\n#endif\n");


		// Now, start the source file
		File source(File::getCurrentWorkingDirectory().getFullPathName() + "\\" +  baseName + ".cpp");
		if (source.exists()) source.deleteFile();
		source.create();

		// Start by adding some comments to it
		source.appendText("/********************** " + getName() + " dialog definition ****************\n");
		source.appendText("| This file defines " + getName() + " dialog. Created with Pink Lemon!      \n");
		source.appendText("| \n");
		source.appendText("| Feel free to modify. This file is under GPL license.\n");
		source.appendText("\\********************** " + getName() + " dialog definition ***************/\n\n");

		// Need declaration
		source.appendText("#include \"your_path_to_juce.h\"\n");
		source.appendText("#include \"" + header.getRelativePathFrom(header.getParentDirectory()) + "\"\n\n");

		// Okay, start declaring stuff
		// Check if we have comboboxes
		bool bCB = false;
		if (getComponentIndex("ComboBox") != -1)
		{
			// Declare the list of stuffs
			source.appendText("// Define combobox list of items for better access\n");
			i = -1;
			while  ((i = getComponentIndex("ComboBox", i+1)) != -1)
			{
				source.appendText("#define CB_" + arrayComponents[i]->componentName.removeCharacters(" ") + "_ItemNumber\t1\n");
				source.appendText("const char * CB_" + arrayComponents[i]->componentName.removeCharacters(" ") + "Res[] = { ");
				if (arrayComponents[i]->componentText.length()) 
					 source.appendText("\""+ arrayComponents[i]->componentText + "\" }; \n\n");
				else source.appendText("\"DefaultHere\" }; \n\n");
			}
			bCB = true;	
		}

		// Now, the constructor (and destructor)
		source.appendText("///////////////////////////////////////////////////////////////////////\n");
		source.appendText(baseName.removeCharacters(" ") + "::" + baseName.removeCharacters(" ") + "() : \n");
		for (i = 0; i < arrayComponents.size(); i++)
		{
			source.appendText(arrayComponents[i]->componentName.removeCharacters(" ") + "(0), ");
		}
		source.appendText("DialogWindow(T(\"" + getName() + "\"), Colours::" + currentBackground.name + ", false) \n");
		source.appendText("{\n");
		if (bCB) source.appendText("\tint i = 0;\n");
		source.appendText("\tsetSize(" + String(getWidth()) + ", " + String(getHeight()) + ");\n");
		source.appendText("\tsetTopLeftPosition(" + String(getScreenX()) + ", " + String(getScreenY()) + ");\n");
		source.appendText("\tsetVisible (true);\n\tsetResizeable (true);\n\tenableDropShadow(true);\n\tsetOpaque (true);\n\n");
		
		// Create the members now
		for (i = 0; i < arrayComponents.size(); i++)
		{
			source.appendText("\t" + arrayComponents[i]->componentName.removeCharacters(" ") + " = new ");
			source.appendText(ComponentInfoVisitor::getCreationLine(arrayComponents[i]) + "\n");
			source.appendText("\tif (" + arrayComponents[i]->componentName.removeCharacters(" ") + ")\n\t{\n");
			if (arrayComponents[i]->componentType == "TextButton" || arrayComponents[i]->componentType == "ToggleButton")
				source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "->setButtonText(T(\"" +  arrayComponents[i]->componentText + "\"));\n");

			if (arrayComponents[i]->componentType == "ScrollBar" || arrayComponents[i]->componentType == "ColourSelector" || arrayComponents[i]->componentType == "ResizerComponent")
			{
				source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "->setName(\"" + arrayComponents[i]->componentName.removeCharacters(" ") + "\");\n");
			}

			source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "->setSize(" + String(arrayComponents[i]->size.getX()) + ", " + String(arrayComponents[i]->size.getY()) + ");\n");
			source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "->setTopLeftPosition(getChildPosition(" + arrayComponents[i]->componentName.removeCharacters(" ") + ").getX(), getChildPosition(" + arrayComponents[i]->componentName.removeCharacters(" ") + ").getY());\n");
			if (arrayComponents[i]->componentLabel.length())	
			{
				source.appendText("\t\tLabel * " + arrayComponents[i]->componentName.removeCharacters(" ") + "Label = new Label(String::empty, \"" + arrayComponents[i]->componentLabel + "\");\n");
				source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "Label->attachToComponent(" + arrayComponents[i]->componentName.removeCharacters(" ") + ", " + String(arrayComponents[i]->componentLabelLeft ? "true" : "false") + ");\n");
				source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "Label->setVisible(true);\n");
			}
	
			if (arrayComponents[i]->componentType == "ComboBox")
			{
				source.appendText("\t\tfor (i = 0; i < CB_" + arrayComponents[i]->componentName.removeCharacters(" ") + "_ItemNumber; i++)\n");
				source.appendText("\t\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "->addItem (CB_" + arrayComponents[i]->componentName.removeCharacters(" ") + "Res[i], i);\n");
				source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "->setSelectedId(0);\n");
			}

			if (arrayComponents[i]->componentType == "ScrollBar" || arrayComponents[i]->componentType == "ColourSelector")
			{
				source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "->addChangeListener(this);\n");
			}
			if (arrayComponents[i]->componentType == "TextEditor")
			{
				source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "->addListener(this);\n");
				source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "->setMultiLine(" + String(*(bool*)arrayComponents[i]->options[0] ? "true" : "false") + ");\n");
				source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "->setReadOnly(" + String(*(bool*)arrayComponents[i]->options[2] ? "true" : "false") + ");\n");
			}
			source.appendText("\t\taddAndMakeVisible(" + arrayComponents[i]->componentName.removeCharacters(" ") + ");\n");
			if (arrayComponents[i]->componentType != "TextEditor" && arrayComponents[i]->componentType != "Slider" && arrayComponents[i]->componentType != "ColourSelector"
			&&  arrayComponents[i]->componentType != "ResizerComponent" && arrayComponents[i]->componentType != "ScrollBar")
				source.appendText("\t\t" + arrayComponents[i]->componentName.removeCharacters(" ") + "->addActionListener(this);\n");
			source.appendText("\t}\n\n");
		}

		// Great, so now finish constructor and write destructor
		source.appendText("}\n\n");
		source.appendText(baseName.removeCharacters(" ") + "::~" + baseName.removeCharacters(" ") + "()\n");
		source.appendText("{\n\t// This window gets deleted when the user clicks its close button.\n\
\tdeleteAllChildren();\n\n\
\t// When the window is gone, we need to actually quit the app, so\n\
\t// calling this method does just that:\n\
\tJUCEApplication::quit();\n\n\
\t// the content component will get deleted by the DialogWindow base class,\n\
\t// and that will clean up the other demo components.\n}\n\n");

		// Write the paint overload (default calls the dialog window class)
		source.appendText("void " + baseName.removeCharacters(" ") + "::paint(JUCE::Graphics &g)\n{\n\tDialogWindow::paint(g);\n}\n\n");

		// Write the resized overload (for anchoring purpose)
		source.appendText("void " + baseName.removeCharacters(" ") + "::resized()\n{\n\tDialogWindow::resized();\n\n");
		for (i = 0; i < arrayComponents.size(); i++)
		{
			source.appendText("\tif (" + arrayComponents[i]->componentName.removeCharacters(" ") + ") " + arrayComponents[i]->componentName.removeCharacters(" ") + "->setTopLeftPosition(getChildPosition(" + arrayComponents[i]->componentName.removeCharacters(" ") + ").getX(), getChildPosition(" + arrayComponents[i]->componentName.removeCharacters(" ") + ").getY());\n");
		}
		source.appendText("}\n\n");

		// Now write the callbacks (for Text editors first)
		if (getComponentIndex("TextEditor") != -1)
		{
			// Declare the list of stuffs
			source.appendText("// Called when the text changed in text editors\n");
			source.appendText("void " + baseName.removeCharacters(" ") + "::textEditorTextChanged (TextEditor &editor)\n{\n");
			i = getComponentIndex("TextEditor", 0);
			String code = "\tif (&editor == " + arrayComponents[i]->componentName.removeCharacters(" ") + ")\n\t{\n\t\t// TODO : Put your handler here for " + arrayComponents[i]->componentName.removeCharacters(" ") + "\n\t}\n";
			while ((i = getComponentIndex("TextEditor", i+1)) != -1)
			{
				code += "\telse if (&editor == " + arrayComponents[i]->componentName.removeCharacters(" ") + ")\n\t{\n\t\t// TODO : Put your handler here for " + arrayComponents[i]->componentName.removeCharacters(" ") + "\n\t}\n";
			}
			source.appendText(code + "}\n\n");
			source.appendText("// Called when the user presses the return key\n");
			source.appendText("void " + baseName.removeCharacters(" ") + "::textEditorReturnKeyPressed (TextEditor &editor)\n{\n");
			source.appendText(code + "}\n\n");
			source.appendText("// Called when the user presses the escape key\n");
			source.appendText("void " + baseName.removeCharacters(" ") + "::textEditorEscapeKeyPressed (TextEditor &editor)\n{\n");
			source.appendText(code + "}\n\n");
			source.appendText("// Called when the text editor loses focus\n");
			source.appendText("void " + baseName.removeCharacters(" ") + "::textEditorFocusLost (TextEditor &editor)\n{\n");
			source.appendText(code + "}\n\n");
		}

		// Now write the callbacks (for ScrollBar first)
		bool bFirst = true;
		if (getComponentIndex("ScrollBar") != -1 || getComponentIndex("ColourSelector") != -1)
		{
			// Declare the list of stuffs
			source.appendText("// Called when scrollbar has changed position\n");
			source.appendText("void " + baseName.removeCharacters(" ") + "::changeListenerCallback (void *object)\n{\n");
			bFirst = true;
			i = -1;
			String code;
			while ((i = getComponentIndex("ScrollBar", i+1)) != -1)
			{
				if (bFirst)
				{
					code = "\tif (object == &" + arrayComponents[i]->componentName.removeCharacters(" ") + ")\n\t{\n\t\t// TODO : Put your handler here for scrollbar change " + arrayComponents[i]->componentName.removeCharacters(" ") + "\n\t}\n";
					bFirst = false;
				} else
					code += "\telse if (object == &" + arrayComponents[i]->componentName.removeCharacters(" ") + ")\n\t{\n\t\t// TODO : Put your handler here for scrollbar change " + arrayComponents[i]->componentName.removeCharacters(" ") + "\n\t}\n";
			}

			// Now get the ColourSelector component, if any
			i = -1;
			while ((i = getComponentIndex("ColourSelector", i+1)) != -1)
			{
				if (bFirst)
				{
					code = "\tif (object == &" + arrayComponents[i]->componentName.removeCharacters(" ") + ")\n\t{\n\t\t// TODO : Put your handler here for colour selection change " + arrayComponents[i]->componentName.removeCharacters(" ") + "\n\t}\n";
					bFirst = false;
				} else
					code += "\telse if (object == &" + arrayComponents[i]->componentName.removeCharacters(" ") + ")\n\t{\n\t\t// TODO : Put your handler here for colour selection change " + arrayComponents[i]->componentName.removeCharacters(" ") + "\n\t}\n";
			}

			source.appendText(code + "}\n\n");
		}

		// And for other components too
		source.appendText("// Called when an action happened\n");
		source.appendText("void " + baseName.removeCharacters(" ") + "::actionListenerCallback(const String &m)\n{\n");
		bFirst = true;
		for (i = 0; i < arrayComponents.size(); i++)
		{
			if (arrayComponents[i]->componentType != "TextEditor" && arrayComponents[i]->componentType != "Slider" && arrayComponents[i]->componentType != "ColourSelector"
			&&  arrayComponents[i]->componentType != "ResizerComponent" && arrayComponents[i]->componentType != "ScrollBar")
			{
				if (bFirst)
				{
					if (arrayComponents[i]->componentType != "HyperlinkButton")
						source.appendText("\tif (m == \"" + arrayComponents[i]->componentName.removeCharacters(" ") + "\")\n\t{\n");
					else
						source.appendText("\tif (m == \"" + arrayComponents[i]->componentName + "\")\n\t{\n");

					bFirst = false; 
				}
				else
				{
					if (arrayComponents[i]->componentType != "HyperlinkButton")
						source.appendText("\telse if (m == \"" + arrayComponents[i]->componentName.removeCharacters(" ") + "\")\n\t{\n");
					else
						source.appendText("\telse if (m == \"" + arrayComponents[i]->componentName + "\")\n\t{\n");
				}

				source.appendText("\t\t// TODO : Put your " + arrayComponents[i]->componentName.removeCharacters(" ") + " action here\n\t}\n");
			}
		}
		source.appendText("\tDialogWindow::actionListenerCallback(m);\n");
		source.appendText("}\n\n");

		// Add the anchoring stuff
		source.appendText("// Returns the position with the correct anchoring applied\n");
		source.appendText("integerPoint " + baseName.removeCharacters(" ") + "::getChildPosition(Component * child)\n{\n");
		source.appendText("\tintegerPoint pt(0, 0);\n\tif (child == 0) return pt;\n");
		// Let the fun goes on
		bFirst = true;
		for (i = 0; i < arrayComponents.size(); i++)
		{
			if (bFirst)
			{
				if (arrayComponents[i]->componentType != "HyperlinkButton")
					source.appendText("\tif (child->getName() == \"" + arrayComponents[i]->componentName.removeCharacters(" ") + "\")\n\t{\n"); 			
				else
					source.appendText("\tif (child->getName() == \"" + arrayComponents[i]->componentName + "\")\n\t{\n"); 			

				bFirst = false;
			} else
			{
				if (arrayComponents[i]->componentType != "HyperlinkButton")
					source.appendText("\telse if (child->getName() == \"" + arrayComponents[i]->componentName.removeCharacters(" ") + "\")\n\t{\n"); 			
				else
					source.appendText("\telse if (child->getName() == \"" + arrayComponents[i]->componentName + "\")\n\t{\n"); 			
			}

			// Now set the point with the right anchoring
			source.appendText("\t\tpt.setXY(");
			//TODO: Made floating anchor working (this requires tracking the resizing side for undoing DialogWindow default top left dialog)
			if (arrayComponents[i]->anchorType & ComponentInfo::AnchorRight)
				source.appendText("getWidth() - child->getWidth() - " + String(getWidth() - (arrayComponents[i]->size.getX() + arrayComponents[i]->position.getX())) + ", ");
			else
				source.appendText(String(arrayComponents[i]->position.getX()) + ", ");
			if (arrayComponents[i]->anchorType & ComponentInfo::AnchorBottom)
				source.appendText("getHeight() - child->getHeight() - " + String(getHeight() - (arrayComponents[i]->size.getY() + arrayComponents[i]->position.getY())) + ");\n");
			else
				source.appendText(String(arrayComponents[i]->position.getY()) + ");\n");

			source.appendText("\t\treturn pt;\n\t}\n");
		}
		source.appendText("\tjassertfalse\t//This method has been called with an unknown child, please add the code for it\n");
		source.appendText("\treturn pt;\n}\n\n");

	} catch (...)
	{
	}
}


bool Window::serialize(const String & fileName, const bool & bReading)
{
	if (bReading)
	{
		// Read the file now
		File in(File::getCurrentWorkingDirectory().getFullPathName() + "\\" + fileName);
		if (!in.exists()) return false;
		XmlDocument myDocument (in);
		XmlElement* mainElement = myDocument.getDocumentElement();

		// Validate the file
		if (!mainElement->hasTagName("PinkLemon"))
			return false;

		for (int i = 0; i < arrayComponents.size(); i++)
		{
			removeChildComponent(arrayComponents[i]->object);
			delete arrayComponents[i]->object;
		}
		arrayComponents.clear(true);
		// Read our data first
		XmlElement * dialog = mainElement->getChildByName("dialog");
		if (!dialog) return false;
		
		// Read the main dialog stuff first
		iTopLeftX	= dialog->getIntAttribute("x");
		iTopLeftY	= dialog->getIntAttribute("y");
		iWidth		= dialog->getIntAttribute("width");
		iHeight		= dialog->getIntAttribute("height");
		gridStep    = dialog->getIntAttribute("grid");
		String name = dialog->getStringAttribute("name");
		String colour = dialog->getStringAttribute("theme");


		// Now, apply them
		setTopLeftPosition(iTopLeftX, iTopLeftY);
		setSize(iWidth, iHeight);
		setName(name);
		
		// Ugly colour changing hack
		changeBackgroundColor(colour);

		// Load the components now
		ComponentInfo * ci = 0;
		XmlElement * el = dialog->getFirstChildElement();
		while (el != 0)
		{
			if (el->hasTagName("component"))
			{
				ci = ComponentInfoVisitor::serialize(el);
				if (ci)
				{
					addComponent(ci);
					if (ci->label) deleteAndZero(ci->label);

					if (ci->componentLabel.length() )
					{
						ci->label = new Label(String::empty, ci->componentLabel);
						ci->label->attachToComponent(ci->object, ci->componentLabelLeft);
						ci->label->setVisible(true);
					} 
				}

			}

			el = el->getNextElement();
		}

		// And repaint
		repaint();
		return true;
	}
	else
	{	
		// Save mode
		XmlElement mainDocument("PinkLemon");

		// First save ourself
		XmlElement * dialog = new XmlElement("dialog");
		if (!dialog) return false;
		dialog->setAttribute("x", iTopLeftX);
		dialog->setAttribute("y", iTopLeftY);
		dialog->setAttribute("width", iWidth);
		dialog->setAttribute("height", iHeight);
		dialog->setAttribute("name", getName());
		dialog->setAttribute("grid", gridStep);
		dialog->setAttribute("theme", currentBackground.name);


		// Now, save the components
		for (int i = 0; i < arrayComponents.size(); i++)
		{
			dialog->addChildElement(ComponentInfoVisitor::serialize(arrayComponents[i]));
		}

		mainDocument.addChildElement(dialog);

		String content = mainDocument.createDocument(String::empty);
		File out(File::getCurrentWorkingDirectory().getFullPathName() + "\\" + fileName);
		if (out.exists()) out.deleteFile();
		out.appendText(content);

		return true;
	}
}

