#ifndef h_CPP_Window_CPP_h
#define h_CPP_Window_CPP_h


class Dialog;

struct ComponentInfo
{
	enum
	{
		AnchorLeft		= 1,
		AnchorRight		= 2,
		AnchorTop	    = 4,
		AnchorBottom    = 8,
		AnchorFloating  = 16,
	};

	// The component type
	String			componentType;
	// The component name
	String			componentName;
	// The component default text
	String			componentText;
	// The component label
	String			componentLabel;
	// The label is on left position switch
	bool			componentLabelLeft;
	// The anchor type
	int				anchorType;
	// The anchor component (null for dialog window)
	ComponentInfo *	anchor;
	// The component position (relative to the anchor)
	integerPoint	position;
	// The component size
	integerPoint	size;
	// Its linked component
	Component *		object;
	// Its linked label
	Label	  *		label;
	// The additionnal component
	VoidArray		options;

	ComponentInfo() : componentType(String::empty), componentName("Cmp_"), componentText(String::empty), componentLabel(String::empty), componentLabelLeft(true), anchorType(AnchorLeft | AnchorTop), anchor(0), position(100, 100), size(100, 30), object(0), label(0) {}

};

struct ComponentInfoVisitor
{
	static Component *	createComponent(ComponentInfo * ci); 
	static bool			startDraggingComponent(ComponentInfo * ci);
	static bool			stopDraggingComponent(ComponentInfo * ci);
	static int			createAndTrackPopupMenu(ComponentInfo * ci, PopupMenu * pClrMenu = 0);
	static const String getCreationLine(ComponentInfo * ci);
	static XmlElement * serialize(ComponentInfo * ci);
	static ComponentInfo * serialize(XmlElement * e);
};

struct ColourUnion
{
    Colour colour;
	String name;

	ColourUnion(const Colour & clr = Colours::black, const String & _name = "black") : colour(clr), name(_name) {}
};

//==============================================================================
class Window  : public DialogWindow
{
	// Callbacks
public:
	virtual	void  keyPressed(const KeyPress & key);
	virtual void  modifierKeysChanged (const ModifierKeys & modifiers); 

	// Members
private:
	// The main dialog pointer
	Dialog *					theDialog;
	// The component info stuff
	OwnedArray<ComponentInfo>	arrayComponents;
	// The command key state
	bool						commandDown;
	// Grid definition
	int							gridStep;
	// Current background color
	ColourUnion					currentBackground;


public:
	// Constructor
    Window(Dialog * dlg);
	// Destructor
    virtual ~Window();

public:
	// Overriding the paint method
    virtual void paint (Graphics& g);
	// And the resized method
	virtual void resized();
	// And the moved method
	virtual void moved();
	// We want to be informed when we are gaining the focus (so we can easily set the z-order of childrens)
	virtual void focusGained (FocusChangeType cause);
	// We want to be informed when we are gaining the focus (so we can easily set the z-order of childrens)
	virtual void broughtToFront();

	void changeBackgroundColor(const String & name);

	void addComponent(ComponentInfo * ci);
	ComponentInfo * findComponent(const integerPoint & pt, const int & deviation = 0);
	int getComponentIndex(const String & type, const int & start = 0);
	ComponentInfo * draggedComponent;
	ComponentInfo * resizingComponent;
	integerPoint    resizingOrigin;
	integerPoint	resizingOriginSize;
	integerPoint	draggedSrc;

	int highlightSide;

	// Mouse listener
public:
	virtual void  mouseUp (const MouseEvent &e);
	virtual void  mouseDrag (const MouseEvent &e);
	virtual void  mouseDown (const MouseEvent &e);
	virtual void  mouseMove (const MouseEvent &e);

	// Initial positions
public:
	// Initial top left position
	int				 iTopLeftX;
	int				 iTopLeftY;
	// Initial width and height;
	int				 iWidth;
	int				 iHeight;
	// Prefilled colour submenu
	PopupMenu		 clrMenu;
	static ColourUnion		 arrayClrUnion[151];

	// Save this dialog as 
public:
	void saveTo(const String & baseName);

	// Serialize this dialog to a Xml file
public:
	bool serialize(const String & fileName, const bool & bReading);
};

#endif